library HmsScriptFunctionsAddon;

uses
  Types,
  SysUtils,
  Classes,
  Windows,
  Variants,
  HmsAddonsIntfTypes in 'HmsAddonsIntfTypes.pas';

{$R *.res}

type
  THmsAddonList = class(TComponent, IHmsAddonList, IHmsAddonTools)
  public
  { IHmsAddonTools }
    function Setup(aOwner: THandle; var aReload: LongBool): HResult; safecall;
    function Update(var aFlags: Integer; var aResult: OleVariant): HResult; safecall;
  { IHmsAddonList }
    function GetCount(var aCount: Integer): HResult; safecall;
    function GetAddonInfo(aIndex: Integer; var aClassID, aInterfaceID: TGUID;
       var aTitle, aDescription, aRequiredVersion, aCheckedOnVersion: OleVariant): HResult; safecall;
    function GetClassObject(const aClassID: TGUID; const aInterfaceID: TGUID; out aObject: OleVariant): HResult; safecall;
    function CanUnloadNow: HResult; safecall;
  end;

  THmsScriptFunctions = class(TComponent, IHmsScriptFunctions)
  protected
    function ExecuteFunction(const aFunctionName: OleVariant; var aFunctionParameters, aFunctionResult: OleVariant): HResult; safecall;
    function GetFunctionCount(var aCount: Integer): HResult; safecall;
    function GetFunctionInfo(aIndex: Integer; var aFunctionName, aFunctionDeclaration,
                             aFunctionCategory, aFunctionDescription: OleVariant): HResult; safecall;
  end;

{ THmsAddonList }

type
  THmsAddonInfo = record
    ClassID: TGUID;
    InterfaceID: TGUID;
    Title: string;
    Description: string;
    RequiredVersion: string;
    CheckedOnVersion: string;
  end;

const
  AddonInfoCount = 1;

  CLASS_HmsScriptFunctions = '{DB54B9AB-926F-45AF-9430-ACC59D30D69C}';

  csAddonTitle = ' ScriptFunctions';

  AddonInfoList: array[0..AddonInfoCount - 1] of THmsAddonInfo =
    ((ClassID : CLASS_HmsScriptFunctions;
      InterfaceID: IID_HmsScriptFunctions;
      Title : csAddonTitle;
      Description: ' ,     ';
      RequiredVersion : '2.06';
      CheckedOnVersion: '2.06'));

{ THmsAddonList }

function THmsAddonList.GetCount(var aCount: Integer): HResult; safecall;
begin
  aCount := AddonInfoCount;
  Result := S_OK
end;

function THmsAddonList.GetAddonInfo(aIndex: Integer; var aClassID, aInterfaceID: TGUID;
       var aTitle, aDescription, aRequiredVersion, aCheckedOnVersion: OleVariant): HResult; safecall;
begin
  if (aIndex >= 0) and (aIndex < AddonInfoCount) then begin
    with AddonInfoList[aIndex] do begin
      aClassID := ClassID;
      aInterfaceID := InterfaceID;
      aTitle := Title;
      aDescription := Description;
      aRequiredVersion := RequiredVersion;
      aCheckedOnVersion := CheckedOnVersion
    end;
    Result := S_OK
  end else
    Result := E_UNEXPECTED
end;

function THmsAddonList.GetClassObject(const aClassID: TGUID; const aInterfaceID: TGUID; out aObject: OleVariant): HResult; safecall;
begin
  Result := E_NOINTERFACE;
  if IsEqualGUID(aInterfaceID, IHmsScriptFunctions) then begin
    aObject := THmsScriptFunctions.Create(nil) as IHmsScriptFunctions;
    Result := S_OK
  end
end;

function THmsAddonList.CanUnloadNow: HResult; safecall;
begin
  Result := S_OK
end;

function THmsAddonList.Setup(aOwner: THandle; var aReload: LongBool): HResult; safecall;
begin
  aReload := MessageBox(aOwner, '  ?', '  ', MB_ICONQUESTION + MB_YESNO) = IDYES;
  Result := S_OK
end;

function THmsAddonList.Update(var aFlags: Integer; var aResult: OleVariant): HResult; safecall;
begin
  aFlags := ufIsInfoMessage;
  aResult := '  ';
  Result := S_OK
end;

const
  ciFunctionCount = 5;

function THmsScriptFunctions.ExecuteFunction(const aFunctionName: OleVariant; var aFunctionParameters, aFunctionResult: OleVariant): HResult; safecall;
begin
  if VarIsArray(aFunctionParameters) then
    aFunctionResult := Format('%s (%s, %s)', [VarToStr(aFunctionName), aFunctionParameters[0], aFunctionParameters[1]])
  else
    aFunctionResult := Format('%s (%s)', [VarToStr(aFunctionName), VarToStr(aFunctionParameters)]);
  Result := S_OK
end;

function THmsScriptFunctions.GetFunctionCount(var aCount: Integer): HResult; safecall;
begin
  aCount := ciFunctionCount;
  Result := S_OK
end;

function THmsScriptFunctions.GetFunctionInfo(aIndex: Integer; var aFunctionName, aFunctionDeclaration, aFunctionCategory,
                                                                  aFunctionDescription: OleVariant): HResult; safecall;
begin
  aFunctionName := Format('ExtFunction_%d', [aIndex + 1]);
  aFunctionDeclaration := Format('function ExtFunction_%d(const aParam1: Integer; const aParam2: string): Integer', [aIndex + 1]);
  aFunctionDescription := '  ' + aFunctionName;
  aFunctionCategory := ' ';
  Result := S_OK
end;

function HmsGetAddonList(out aAddonList: IHmsAddonList): HResult; stdcall;
begin
  aAddonList := THmsAddonList.Create(nil);
  Result := S_OK
end;

exports
  HmsGetAddonList;

end.
